@@ -10,13 +10,13 @@
 #
 # $Project: /Convert-Binary-C $
 # $Author: mhx $
-# $Date: 2007/06/11 20:59:37 +0200 $
-# $Revision: 10 $
+# $Date: 2005/05/23 15:08:55 +0200 $
+# $Revision: 6 $
 # $Source: /token/blproperty.pl $
 #
 ################################################################################
 #
-# Copyright (c) 2002-2007 Marcus Holland-Moritz. All rights reserved.
+# Copyright (c) 2002-2005 Marcus Holland-Moritz. All rights reserved.
 # This program is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself.
 #
@@ -144,7 +144,7 @@ END
           push @optspec, "{ $name, BLPVT_$uctype, sizeof $opt / sizeof $opt\[0], &$opt\[0] }";
         }
         else {
-          push @optspec, "{ $name, BLPVT_$uctype, 0, 0 }";
+          push @optspec, "{ $name, BLPVT_$uctype, 0 }";
         }
       }
 
@@ -236,7 +236,7 @@ const char *bl_property_string(BLProperty property)
 $blp_strings
   };
 
-  if (property < sizeof properties / sizeof properties[0])
+  if (property >= 0 && property < sizeof properties / sizeof properties[0])
     return properties[property];
 
   return NULL;
@@ -248,7 +248,7 @@ const char *bl_propval_string(BLPropValStr propval)
 $blpv_strings
   };
 
-  if (propval < sizeof propvalues / sizeof propvalues[0])
+  if (propval >= 0 && propval < sizeof propvalues / sizeof propvalues[0])
     return propvalues[propval];
 
   return NULL;
